clear all;close all;clc %#ok<CLALL>
opengl hardware
%%
Interactions=NaN(26,40,5); % 26 participants x 40 trials x 5 indexes
for p=1:26
    try
        Interactions(p,:,:)=xlsread('Interactions.xlsx',p,'A2:E41'); % 1 = Trial nr, 2 = Condition, 3 = Hand, 4 = Yield, 5 = Step (not used)
    catch error
    end
end
%% Read UNITY data from Unity (stored as .mat file)
load('X') % load X matrix of 26 participants x 40 trials x 10000 sampling indexes x 23 variables
% Variables 2 and 11 represent the position of the vehicles. Variable 23 describes hand-raising
% The rest is not used
%% Process the X matrix into an Xt matrix (26 participants x 40 trials x 1201 sample points x 23 variables)
YieldingObserved=NaN(26,40); % Yielding (1 = no, 2 = yes), as observed from experiment data
YieldingObserved2=Interactions(:,:,4);YieldingObserved2(YieldingObserved2==0)=2; % Yielding (1 = yes, 2 = no), as observed from Interactions.xlsx
% only in 0.2% of the trials there is a congruence

Xt=NaN(26,40,1201,23);
for p=1:26
    disp(p)
    TrialDataCombined=[];
    for t=1:40 % loop over 40 trials
        TrialData=squeeze(X(p,t,:,:));
        TrialDataCombined=[TrialDataCombined;TrialData]; %#ok<AGROW> % append all trial data in one matrix 'TrialDataCombined'
    end
    
    invalid_indexes=find(TrialDataCombined(:,4)==0 | TrialDataCombined(:,13)==0 | isnan(TrialDataCombined(:,4)) | isnan(TrialDataCombined(:,13)) | [NaN;diff(TrialDataCombined(:,1))]==0 | [NaN;diff(TrialDataCombined(:,10))]==0);
    TrialDataCombined(invalid_indexes,:)=[]; %#ok<*SAGROW> % remove invalid indexes
    
    start_indexes=find(TrialDataCombined(:,2)>-110);start_indexes=[start_indexes(1);start_indexes(find(diff(start_indexes)>1)+1)]; % extract start_indexes of starts of trials
    
    % Participants 1, 8, and 10 contain a pause in the data logging, so we have to apply a manual correction
    if p==1;start_indexes(23:40)=start_indexes(22:39);start_indexes(22)=NaN;end
    if p==8;start_indexes(4:40)=start_indexes(3:39);start_indexes(3)=NaN;end
    if p==10;start_indexes(11:40)=start_indexes(10:39);start_indexes(10)=NaN;end
    
    yielding=NaN(40,1);
    
    for t=1:length(start_indexes) % loop over all starts of trials
        if ~isnan(start_indexes(t))
            if TrialDataCombined(start_indexes(t)+400,2)>-49.5 || TrialDataCombined(start_indexes(t)+1079,2)<-110  % if there is an anomaly in the timing of the approach of vehicle 1, then set the index to NaN
                start_indexes(t)=NaN;
            end
        end
        
        if ~isnan(start_indexes(t))
            YieldingObserved(p,t)=1*(TrialDataCombined(start_indexes(t)+1078,11)<0) + 2*(TrialDataCombined(start_indexes(t)+1078,11)>0); % 1 = yielding, 2 = nonyielding (calculated based on where the second vehicle is)
            Xt(p,t,1:1079,:)=TrialDataCombined(start_indexes(t):start_indexes(t)+1078,:);
        end
    end
end
%% Figure 2. The approaching vehicles' distance to pedestrian
figure;hold on
rectangle('position',[5.80 -20 1.43 140],'facecolor',[.9 .9 .9],'Linestyle','none'); % Plot gray area from 5.80 to 7.23 s

counter=0;
for t=1:40
    if Interactions(1,t,2)<3 % 1 = Baseline, 2 = eHMI, 3 = Hand, 4 = Combination
        figure(1);hold on
        if counter==0
            plot(linspace(0,11.975,1079), -squeeze(Xt(1,t,1:1079,2)),'k','Linewidth',3); % Vehicle 1 (nonyielding)
            counter=1;
        end
        if YieldingObserved(1,t)==2 && counter==1
            plot(linspace(0,11.975,1079), -squeeze(Xt(1,t,1:1079,11)),'r','Linewidth',3); % Vehicle 2 (nonyielding)
            d=find(squeeze(Xt(1,t,1:1079,11))>-50 & squeeze(Xt(1,t,1:1079,11))<-30);min(d),max(d)
            counter=2;
        end
        if YieldingObserved(1,t)==1 && counter==2
            plot(linspace(0,11.975,1079), -squeeze(Xt(1,t,1:1079,11)),'b:','Linewidth',3); % Vehicle 2 (yielding)
            counter=3;
        end
    end
end

legend('Lead vehicle (nonyielding)','Target vehicle (nonyielding)','Target vehicle (yielding)')
grid on
ylabel('Distance to pedestrian (m)')
xlabel('Time (s)')
h=findobj('FontName','Helvetica');set(h,'FontSize',24,'Fontname','Arial');
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01])
%%
Post_trial=xlsread('Posttrial.xlsx','C2:H1241'); % 1040 x 6 matrix
% 1. Participant number
% 2. Trial number
% 3. Difficulty of predicting the AV behavior
% 4. Assurance of being seen by the AV
% 5. Condition (1 = Baseline, 2 = eHMI, 3 = Hand, 4 = Combination)
% 6. Yielding

YieldingMatrixIfHand=NaN(26,40); % Preallocate 26 participants x 40 trials matrix
for p=1:26 % Loop over 26 participants
    YieldingMatrixIfHand(p,:)=Post_trial(Post_trial(:,1)==p,6); % Store experimenter files describing the programmed yielding behavior (i.e., the behavior of the vehicle if the hand would be raised)
end
%% Figure 3. Boxplot depicting the percentage of trials with hand gesture usage per condition
Condition=Interactions(:,:,2); % 1 = Baseline, 2 = eHMI, 3 = Hand, 4 = Combination
Condition(5,1:10)=1;Condition(5,11:20)=3;Condition(5,21:30)=2;Condition(5,31:40)=4; % Impute manually for participant 5
% Condition is a 26 participant x 40 trial matrix listing the conditions (1 = Baseline, 2 = eHMI, 3 = Hand, 4 = Combination)

HandInInterval=double(sum(Xt(:,:,523:652,23),3)>130); % 26 x 40 matrix describing whether hand was raised in interval (1 = Yes, 0 = No)
HandInInterval(isnan(YieldingObserved))=NaN; % If there was no data in that trial, then set HandInInterval to NaN
HandMustHaveBeenRaised    = YieldingObserved==1 & YieldingMatrixIfHand==1 & Condition>2.5; % (1) Vehicle is observed to yield (1 = Yielding),         (2) Trial is a yielding trial (if hand would be raised), in (3) Hand or Combination trials
HandMustHaveNotBeenRaised = YieldingObserved==2 & YieldingMatrixIfHand==1 & Condition>2.5; % (1) Vehicle is observed not to yield (2 = NonYielding),  (2) Trial is a yielding trial (if hand would be raised), in (3) Hand or Combination trials
E1=HandMustHaveBeenRaised & HandInInterval==0;    % Hand must have been raised, but hand raise was not observed
E2=HandMustHaveNotBeenRaised & HandInInterval==1; % Hand must not have been raised, but hand raise was observed
YieldingObserved(E1+E2==1)=NaN; % If the yielding behavior does not match the hand gesture, then remove that trial (this is less than 1% of trials, presumably due to an experimental/logging error)
Condition(isnan(YieldingObserved))=NaN;

[NTH,NT]=deal(NaN(26,4)); % Preallocate matrix (26 participants x 4 experimental conditions)

for p=1:26 % Loop over 26 participants
    for c=1:4 % Loop over 4 experimental conditions
        NTH(p,c)=sum(HandInInterval(p,:)==1 & Condition(p,:)==c); % Number of times the participant raised hand in that condition
        NT(p,c)=sum(Condition(p,:)==c); % Number of trials for the participant in that condition
    end
end

figure
notBoxPlot4b(100*NTH./NT,1);hold on;box on;grid on
ylabel('Percentage of trials with hand gesture')
set(gca,'xticklabel',{'Baseline','eHMI','Hand','Combination'})
set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'xtick',1:4,'xlim',[0.5 4.6])
xtickangle(45)
h=findobj('FontName','Helvetica');set(h,'FontSize',24,'Fontname','Arial');

disp('SDs in hand gesture usage (means are shown in boxplot)')
disp(std(100*NTH./NT))
%% Figure 4. Percentage of trians in the Hand and Combination conditions in which participants had raised their hand as a function of elapsed time
for Y=1:2 % 1 = Yielding, 2 = Non-yielding
    counter=0;
    ALLC=NaN(26,4,10,1079);
    ALLCCUM=NaN(26,4,10,1079);
    ALLCRELEASE=NaN(26,4,10);
    for p=1:26
        for c=3:4 % 1 = Baseline, 2 = eHMI, 3 = Hand, 4 = Combination
            
            AllC=(squeeze(Xt(p,Condition(p,:)==c & YieldingMatrixIfHand(p,:)==Y,1:1079,23))); % all hand-raise trials for participant p, condition c
            if size(AllC,1)>size(AllC,2)
                AllC=transpose(AllC);
            end
            
            AllCCum=zeros(size(AllC,1),130);
            for i=1:size(AllCCum,1) % loop over a maximum of 5 trials for that person & condition
                first_hand_use=find(AllC(i,523:652)==2,1,'first'); % find first moment of hand use
                if ~isempty(first_hand_use)
                    AllCCum(i,first_hand_use:end)=1; % fill with 1s till the end of the trial
                end
            end
            
            AllCRelease=NaN(size(AllC,1),1);
            for i=1:size(AllCCum,1) % loop over a maximum 5 trials for that person & condition
                first_hand_release=find(diff(AllC(i,523:end))==-1,1,'first'); % find first moment of hand release
                if AllCCum(i,end)~=1 % only count releases if the hand was raised in that trial
                    first_hand_release=NaN;
                end
                if ~isempty(first_hand_release)
                    AllCRelease(i)=first_hand_release;
                end
            end
            
            ALLC(p,c,1:size(AllC,1),1:size(AllC,2))=AllC;
            ALLCCUM(p,c,1:size(AllC,1),523:652)=AllCCum;
            ALLCRELEASE(p,c,1:size(AllC,1))=AllCRelease;
            
        end
    end
    figure;hold on
    rectangle('position',[5.80 0 1.43 100],'facecolor',[.9 .9 .9],'Linestyle','none');
    d=100*squeeze(sum(sum(double(ALLC==2),1),3))./(squeeze(sum(sum(double(ALLC==1),1),3))+squeeze(sum(sum(double(ALLC==2),1),3)));
    plot(linspace(0,11.975,1079),d(3,:)','k','Linewidth',3) % Hand condition
    plot(linspace(0,11.975,1079),d(4,:)','r','Linewidth',3) % Combination condition
    d=100*squeeze(sum(sum(double(ALLCCUM==1),1),3))./(squeeze(sum(sum(double(ALLCCUM==1),1),3))+squeeze(sum(sum(double(ALLCCUM==0),1),3)));
    plot(linspace(0,11.975,1079),d(3,:)','k:','Linewidth',3) % Hand condition (cumulative)
    plot(linspace(0,11.975,1079),d(4,:)','r:','Linewidth',3) % Combination condition (cumulative)
    legend('Hand','Combination','Hand (cumulative)','Combination (cumulative)')
    grid on
    ylabel('Percentage of trials with hand gesture')
    xlabel('Time (s)')
    h=findobj('FontName','Helvetica');set(h,'FontSize',24,'Fontname','Arial');
    set(gca,'TickDir','out','LooseInset',[0.01 0.01 0.01 0.01],'ylim',[0 80])
    
    disp(' ')
    
    if Y==1
        disp('****Trials in which the AV could yield')
    else
        disp('****Trials in which the AV did not yield (i.e., other trials)')
    end
    d=squeeze(nanmean(ALLCRELEASE,3))*mean(diff(linspace(0,11.975,1079))); % Samling index of hand release multiplied by sampling time
    v1=d(:,3); % Hand condition
    v2=d(:,4); % Combination condition
    removepp=find(isnan(v1+v2));
    v1(removepp)=[];
    v2(removepp)=[];
    
    disp(['Release time - Hand: Mean (SD): '  sprintf('%0.2f',mean(v1)) ' (' sprintf('%0.2f',std(v1)) ')'])
    disp(['Release time - Combination: Mean (SD): '  sprintf('%0.2f',mean(v2)) ' (' sprintf('%0.2f',std(v2)) ')'])
    [~,p,~,stats] = ttest(v1,v2);
    disp(['ttest: t(' num2str(stats.df) ') = ' sprintf('%0.2f',stats.tstat) ', p = ' sprintf('%0.3f',p)])
end
%% Figure 5. Mean responses to posttrial questions
[QDiff,QSeen]=deal(NaN(26,40));

for p=1:26 % loop over participants
    QDiff(p,:)=Post_trial(Post_trial(:,1)==p,3); % Store data for 'on a scale from 0 to 10, how difficult was it for you to predict the behavior of the car, where 0 is not difficult at all, and 10 is very difficult'
    QSeen(p,:)=Post_trial(Post_trial(:,1)==p,4); % Store data for 'on a scale from 0 to 10, how sure were you that the car would see you, where 0 is not sure at all and 10 is completely sure?'
end
QDiff(isnan(YieldingObserved))=NaN;
QSeen(isnan(YieldingObserved))=NaN;

[QDiffcY,QSeencY,QDiffcNY,QSeencNY]=deal(NaN(26,4));
for p=1:26
    for c=1:4
        QDiffcY(p,c)=nanmean(QDiff(p,Condition(p,:)==c & YieldingObserved(p,:)==1));
        QSeencY(p,c)=nanmean(QSeen(p,Condition(p,:)==c & YieldingObserved(p,:)==1));
        QDiffcNY(p,c)=nanmean(QDiff(p,Condition(p,:)==c & YieldingObserved(p,:)==2));
        QSeencNY(p,c)=nanmean(QSeen(p,Condition(p,:)==c & YieldingObserved(p,:)==2));
    end
end

confidenceinterval1=within_subject_ci(QDiffcY);
confidenceinterval2=within_subject_ci(QSeencY);
confidenceinterval3=within_subject_ci(QDiffcNY);
confidenceinterval4=within_subject_ci(QSeencNY);

Conditions={'Baseline','eHMI', 'Hand', 'Combination'};

figure
subplot(2,2,1)
ydt=nanmean(QDiffcY);
ydts=nanstd(QDiffcY);
barh(ydt,'facecolor',[.5 .5 .5]);box off;hold on
for i=1:4;text(0+10/100,i,[sprintf('%0.2f',ydt(i)) ' (' sprintf('%0.2f',ydts(i)) ')'],'color',[1 1 1]);end
errorbar(ydt, 1:length(ydt), confidenceinterval1, '.b','horizontal','handlevisibility','off','linewidth',2)
grid on
set(gca,'ytick',1:4,'yticklabel',Conditions, 'xlim',[0 10],'tickdir','out','ticklength',[0.01 0.00],'xtick',0:2:100,'ydir','reverse','ylim',[.4 4.6])
title('Difficulty of predicing the AV''s behavior (Yielding AVs)')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',14)
xlabel('Score (0-10)')
set(gca,'pos',[0.08 0.60 0.38 0.35])

subplot(2,2,2)
ydt=nanmean(QSeencY);
ydts=nanstd(QSeencY);
barh(ydt,'facecolor',[.5 .5 .5]);box off;hold on
for i=1:4;text(0+10/100,i,[sprintf('%0.2f',ydt(i)) ' (' sprintf('%0.2f',ydts(i)) ')'],'color',[1 1 1]);end
errorbar(ydt, 1:length(ydt), confidenceinterval2, '.b','horizontal','handlevisibility','off','linewidth',2)
grid on
set(gca,'ytick',1:4,'yticklabel',Conditions, 'xlim',[0 10],'tickdir','out','ticklength',[0.01 0.00],'xtick',0:2:100,'ydir','reverse','ylim',[.4 4.6])
title('Sureness of being seen by the AV (Yielding AVs)')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',14)
xlabel('Score (0-10)')
set(gca,'pos',[0.60 0.60 0.38 0.35])

subplot(2,2,3)
ydt=nanmean(QDiffcNY);
ydts=nanstd(QDiffcNY);
barh(ydt,'facecolor',[.5 .5 .5]);box off;hold on
for i=1:4;text(0+10/100,i,[sprintf('%0.2f',ydt(i)) ' (' sprintf('%0.2f',ydts(i)) ')'],'color',[1 1 1]);end
errorbar(ydt, 1:length(ydt), confidenceinterval3, '.b','horizontal','handlevisibility','off','linewidth',2)
grid on
set(gca,'ytick',1:4,'yticklabel',Conditions, 'xlim',[0 10],'tickdir','out','ticklength',[0.01 0.00],'xtick',0:2:100,'ydir','reverse','ylim',[.4 4.6])
title('Difficulty of predicing the AV''s behavior (Nonyielding AVs)')
xlabel('Score (0-10)')
set(gca,'pos',[0.08 0.08 0.38 0.35])

subplot(2,2,4)
ydt=nanmean(QSeencNY);
ydts=nanstd(QSeencNY);
barh(ydt,'facecolor',[.5 .5 .5]);box off;hold on
for i=1:4;text(0+10/100,i,[sprintf('%0.2f',ydt(i)) ' (' sprintf('%0.2f',ydts(i)) ')'],'color',[1 1 1]);end
errorbar(ydt, 1:length(ydt), confidenceinterval4, '.b','horizontal','handlevisibility','off','linewidth',2)
grid on
set(gca,'ytick',1:4,'yticklabel',Conditions, 'xlim',[0 10],'tickdir','out','ticklength',[0.01 0.00],'xtick',0:2:100,'ydir','reverse','ylim',[.4 4.6])
title('Sureness of being seen by the AV (Nonyielding AVs)')
xlabel('Score (0-10)')
set(gca,'pos',[0.60 0.08 0.38 0.35])
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',18)

v1=QDiffcNY(:,1); % Baseline condition, nonyielding AVs
v2=QDiffcNY(:,3); % Hand condition, nonyielding AVs
disp(' ')
disp(['Difficulty of predicting AV''s behavior (nonyielding AVs) - Baseline: Mean (SD): '  sprintf('%0.2f',mean(v1)) ' (' sprintf('%0.2f',std(v1)) ')'])
disp(['Difficulty of predicting AV''s behavior (nonyielding AVs) - Hand: Mean (SD): '  sprintf('%0.2f',mean(v2)) ' (' sprintf('%0.2f',std(v2)) ')'])
[~,p,~,stats] = ttest(v1,v2);
disp(['ttest: t(' num2str(stats.df) ') = ' sprintf('%0.2f',stats.tstat) ', p = ' sprintf('%0.3f',p)])

v1=QDiffcY(:,1); % Baseline condition, yielding AVs
v2=QDiffcY(:,2); % eHMI condition, yielding AVs
disp(' ')
disp(['Difficulty of predicting AV''s behavior (yielding AVs) - Baseline: Mean (SD): '  sprintf('%0.2f',mean(v1)) ' (' sprintf('%0.2f',std(v1)) ')'])
disp(['Difficulty of predicting AV''s behavior (yielding AVs) - eHMI: Mean (SD): '  sprintf('%0.2f',mean(v2)) ' (' sprintf('%0.2f',std(v2)) ')'])
[~,p,~,stats] = ttest(v1,v2);
disp(['ttest: t(' num2str(stats.df) ') = ' sprintf('%0.2f',stats.tstat) ', p = ' sprintf('%0.3f',p)])
%% Table 1. Responses to pre- and postexperiment questions regarding trust in automated vehicles and hand gestures
PrePost=xlsread('PrePostExperiment.xlsx','F2:M27');
for i=1:4 % loop over four questions
    v1=PrePost(:,i); % Preexperiment
    v2=PrePost(:,i+4); % Postexperiment
    disp(' ')
    disp(['Question: ' num2str(i) ' Pre: Mean (SD): '  sprintf('%0.2f',mean(v1)) ' (' sprintf('%0.2f',std(v1)) ')'])
    disp(['Question: ' num2str(i) ' Post: Mean (SD): '  sprintf('%0.2f',mean(v2)) ' (' sprintf('%0.2f',std(v2)) ')'])
    [~,p,~,stats] = ttest(v1,v2);
    disp(['ttest: t(' num2str(stats.df) ') = ' sprintf('%0.2f',stats.tstat) ', p = ' sprintf('%0.3f',p)])
end
%% Table 2
%% Supplementary functions
% Reduced from a script of Rob Campbell
% https://nl.mathworks.com/matlabcentral/fileexchange/26508-notboxplot
function notBoxPlot4b(y,roundingdigits)

x=1:size(y,2);
jitter=0.2;
hold on
for ii=1:size(y,2)
    myPlotter(x(ii),y(:,ii))
end
hold off
    function myPlotter(X,Y)
        jitScale=jitter*1.2;
        for k=1:length(X)
            thisY=Y(:,k);
            
            thisY=thisY(~isnan(thisY));
            thisX=repmat(X(k),1,length(thisY));
            ptch=patch([X(k)-jitScale, X(k)+jitScale, X(k)+jitScale, ...
                X(k)-jitScale],[prctile(Y(:,k),25), ...
                prctile(Y(:,k),25),prctile(Y(:,k),75), ...
                prctile(Y(:,k),75)], 0);
            set(ptch,'edgecolor',[0.8 0.56 0.56],'facecolor',[1 0.7 0.7])
            plot([X(k)-jitScale,X(k)+jitScale], ...
                [median(thisY),median(thisY)], ...
                '-r','linewidth', 2);
            thisX=violaPoints(thisX,thisY);
            plot(thisX,thisY,'o','color',[0 0 0], ...
                'markerfacecolor','none', 'markersize', 8, ...
                'linewidth', 2);
            plot(X(k),nanmean(Y),'x','linewidth',4,'markersize', 20, ...
                'markeredgecolor',[0 255 100]/255)
            
            if roundingdigits==0
                text(X(k)+0.25,nanmean(Y),sprintf('%0.0f',nanmean(Y)))
            elseif roundingdigits==1
                if nanmean(Y)==0 && ii<=2
                    text(X(k)+0.25,2.5,sprintf('%0.1f',nanmean(Y)))
                elseif nanmean(Y)==0 && ii==7
                    text(X(k)+0.25,6.25,sprintf('%0.1f',nanmean(Y)))
                else
                    text(X(k)+0.25,nanmean(Y),sprintf('%0.1f',nanmean(Y)))
                end
            elseif roundingdigits==2
                text(X(k)+0.25,nanmean(Y),sprintf('%0.2f',nanmean(Y)))
            end
        end
        
    end
    function X=violaPoints(X,Y)
        [counts,~,bins]=histcounts(Y,10);
        inds=find(counts~=0);
        counts=counts(inds);
        Xr=X;
        for jj=1:length(inds)
            tWidth=jitter*(1-exp(-0.1*(counts(jj)-1)));
            xpoints=linspace(-tWidth*0.8,tWidth*0.8,counts(jj));
            Xr(bins==inds(jj))=xpoints;
        end
        X=X+Xr;
    end
end